var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Injectable } from '@angular/core';
import { merge } from 'lodash';
import { SnapshotConfig } from './snapshot-config';
import { SnapshotFileHandler } from './snapshot-files';
import { SnapshotReporter } from './reporter';
import { diffSnapshots } from './snapshot-differ';
var SnapshotRunner = (function () {
    function SnapshotRunner(config, fileHandler, reporter) {
        var _this = this;
        this.config = config;
        this.fileHandler = fileHandler;
        this.reporter = reporter;
        this.specCounter = 0;
        this.encounteredSnapshots = {};
        this.loadedSnapshots = {};
        this.activeFilePath = null;
        reporter.specStarted$.subscribe(function (spec) { return _this.setSpecDescription(spec); });
    }
    SnapshotRunner.prototype.setSpecDescription = function (spec) {
        this.specDescription = spec.fullName;
        this.specCounter = 0;
    };
    SnapshotRunner.prototype.getCurrentSnapshot = function () {
        var notFound = { found: false };
        if (!Boolean(this.loadedSnapshots[this.specDescription])) {
            return notFound;
        }
        var specSnapshots = this.loadedSnapshots[this.specDescription];
        if (!Boolean(specSnapshots[this.specCounter])) {
            return notFound;
        }
        var currentSnapshot = specSnapshots[this.specCounter];
        return { found: true, snapshot: currentSnapshot };
    };
    SnapshotRunner.prototype.encounterSnapshot = function (snapshot) {
        if (this.config.failOnSnapshotDiscovery) {
            throw new Error("Snapshot missing for \"" + this.specDescription + "\"");
        }
        if (!Boolean(this.encounteredSnapshots[this.specDescription])) {
            this.encounteredSnapshots[this.specDescription] = (_a = {},
                _a[this.specCounter] = snapshot,
                _a);
        }
        else {
            this.encounteredSnapshots[this.specDescription][this.specCounter] = snapshot;
        }
        var _a;
    };
    SnapshotRunner.prototype.loadSnapshots = function (file) {
        if (this.activeFilePath !== null) {
            throw new Error("Previous snapshots have not been saved");
        }
        this.loadedSnapshots = this.fileHandler.load(file);
        this.encounteredSnapshots = {};
        this.activeFilePath = file;
    };
    SnapshotRunner.prototype.compareSnapshot = function (fixture) {
        if (this.activeFilePath === null) {
            throw new Error("No snapshots have been loaded");
        }
        if (fixture.toJSON) {
            fixture = fixture.toJSON();
        }
        if (typeof fixture !== 'string') {
            fixture = JSON.stringify(fixture, null, 2);
        }
        var _a = this.getCurrentSnapshot(), found = _a.found, snapshot = _a.snapshot;
        if (!found) {
            this.encounterSnapshot(fixture);
            ++this.specCounter;
            return { pass: true };
        }
        ++this.specCounter;
        return diffSnapshots(snapshot, fixture);
    };
    SnapshotRunner.prototype.saveSnapshots = function () {
        var snapshots = merge({}, this.loadedSnapshots, this.encounteredSnapshots);
        this.fileHandler.save(this.activeFilePath, snapshots);
        this.activeFilePath = null;
    };
    return SnapshotRunner;
}());
SnapshotRunner = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [SnapshotConfig,
        SnapshotFileHandler,
        SnapshotReporter])
], SnapshotRunner);
export { SnapshotRunner };
export var PROVIDERS = [
    SnapshotRunner
];
//# sourceMappingURL=runner.js.map